%This case study considers portfolio optimization problem with the average
%gain objective function, CVaR constraint, and nonlinear transaction cost
%depending upon the total dollar value of the bought/sold assets
%%------------------------------------------------------------------------
clc
clear;
format long;

%------------------------------------------------
%Solve the problem    problem_1_Trans_Costs   |
%------------------------------------------------

problem_name = 'problem_1_Trans_Costs_Toolbox.mat';

%Load the problem and data:
load(['.\' problem_name]);
tbpsg_export_to_workspace(toolboxstruc_arr);

%Define problem statement:
problem_statement = sprintf('%s\n',...
'maximize',...
'Objective: objective_expected_gain_1, linearize = 1',...
'  avg_g(matrix_loss_1)',...
'Constraint: <= 0, linearize = 1',...
'  polynom_abs(matrix_polynom_abs_1)',...
'  linear(matrix_initial_prices)',...               
'Constraint: constraint_on_CVaR_1, <= 1000, linearize = 1',...
'  cvar_risk(0.90000000, matrix_loss_1)',...
'Box: >= point_lowerbounds, <= point_upperbounds',...
'');
%Uncomment the following line to open the problem in Toolbox Window:
%tbpsg_toolbox(problem_statement,toolboxstruc_arr);

%Optimize the problem problem_1_Trans_Costs 
[solution_str1, outargstruc_arr1] = tbpsg_run(problem_statement,toolboxstruc_arr);  

%extract optimal point for the problem problem_1_Trans_Costs
point_variables = tbpsg_optimal_point_vars(solution_str1, outargstruc_arr1);
X_opt1 = tbpsg_optimal_point_data(solution_str1, outargstruc_arr1)';

fprintf('solution: %s\n', solution_str1);

clear D iargstruc_arr outargstruc_arr problem_name problem_statement ...
    solution_str point_variables;

%------------------------------------------------
%Solve the problem    problem_2_Trans_Costs   |
%------------------------------------------------

problem_name = 'problem_2_Trans_Costs_Toolbox.mat';

%Load the problem and data:
load(['.\' problem_name]);
tbpsg_export_to_workspace(toolboxstruc_arr);

%Define problem statement:
problem_statement = sprintf('%s\n',...
'maximize',...
'Objective: objective_expected_gain_2, linearize = 1',...
'  avg_g(matrix_loss_2)',...
'Constraint: <= 1000, linearize = 1',...
'  cvar_risk_2(0.90000000, matrix_loss_2)',...
'Constraint: <= 0',...
'  linear(matrix_budget)',...
'Box: >= point_lowerbounds, <= point_upperbounds',...
'');

%Uncomment the following line to open the problem in Toolbox Window:
%tbpsg_toolbox(problem_statement,toolboxstruc_arr);

%Optimize the problem problem_2_Trans_Costs 
[solution_str2, outargstruc_arr2] = tbpsg_run(problem_statement,toolboxstruc_arr); 

%extract optimal point for the problem problem_2_Trans_Costs
point_variables = tbpsg_optimal_point_vars(solution_str2, outargstruc_arr2);
X_opt2 = tbpsg_optimal_point_data(solution_str2, outargstruc_arr2)';
 
X_opt2_long = X_opt2(1:2:end);
X_opt2_short = X_opt2(2:2:end);

fprintf('solution: %s\n', solution_str2);

clear D iargstruc_arr outargstruc_arr problem_name problem_statement ...
    solution_str point_variables;

%------------------------------------------------
%Solve the problem    problem_3_Trans_Costs   |
%------------------------------------------------

problem_name = 'problem_3_Trans_Costs_Toolbox.mat';

%Load the problem and data:
load(['.\' problem_name]);
tbpsg_export_to_workspace(toolboxstruc_arr);

%Define problem statement:
problem_statement = sprintf('%s\n',...
'maximize',...
'Objective: objective_expected_gain_1, linearize = 1',...
'  avg_g(matrix_loss_1)',...
'Constraint: <= 0',...
'  linear(matrix_initial_prices)',...
'  polynom_abs(matrix_polynom_abs_2)',...
'Constraint: <= 1000, linearize = 1',...
'  cvar_risk(0.90000000, matrix_loss_1)',...
'Box: >= point_lowerbounds, <= point_upperbounds',...
'');

%Uncomment the following line to open the problem in Toolbox Window:
%tbpsg_toolbox(problem_statement,toolboxstruc_arr);

%Optimize the problem problem_3_Trans_Costs 
[solution_str3, outargstruc_arr3] = tbpsg_run(problem_statement,toolboxstruc_arr); 

%extract optimal point for the problem problem_3_Trans_Costs
point_variables = tbpsg_optimal_point_vars(solution_str3, outargstruc_arr3);
X_opt3 = tbpsg_optimal_point_data(solution_str3, outargstruc_arr3)';

fprintf('solution: %s\n', solution_str3);
fprintf('Compare Optimal Solutions for Three Problems');
fprintf('\n\n Component   Problem_1      Problem_2      Problem_2   Problem_3\n');
fprintf('                          long position  short position\n');
for j=1:1:length(point_variables)
    fprintf('%6s%16f%15f%14f%14f\n', point_variables{j},X_opt1(j),X_opt2_long(j), X_opt2_short(j),X_opt3(j));
end

%=======================================================================|
%American Optimal Decisions, Inc. Copyright                             |
%Copyright American Optimal Decisions, Inc. 2007-2016.                 |
%American Optimal Decisions (AOD) retains copyrights to this material.  |
%                                                                       |
%Permission to reproduce this document and to prepare derivative works  |
%from this document for internal use is granted, provided the copyright |
%and No Warranty  statements are included with all reproductions       |
%and derivative works.                                                  |
%                                                                       |
%For information regarding external or commercial use of copyrighted    |
%materials owned by AOD, contact AOD at support@aorda.com.              |
%=======================================================================|
